/*
 * @(#)RoundButton.java	1.00 14th March 1999
 * 
 * Modification Log:
 *
 */

import java.awt.*;
import java.awt.event.*;


/**
 * A simple control that can be used as circular button.
 * Displays a circular light with foreground color with the text centered.
 * Text is underlined on focus.
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 14th March, 1999
 */


public class RoundButton extends RoundLight implements KeyListener, MouseListener, FocusListener, ComponentListener
{
    private String      sText;
    private FontMetrics fm;
    private Dimension   size;
    private int         iWidth, iStartWidth, iStartHeight;
    private boolean     bHasFocus;
    ActionListener      actionListener;  // Post action events to listeners
    
    public RoundButton(String sText)
    {
        addKeyListener(this);
        addMouseListener(this);
        addFocusListener(this);
        addComponentListener(this);
        setText(sText);
    }
    
    public RoundButton()
    {
        addKeyListener(this);
        addMouseListener(this);
        addFocusListener(this);
        addComponentListener(this);
    }
    
    public void setText(String sText)
    {
        this.sText = sText;
    }

    public void paint(Graphics gGraphics)
    {
        super.paint(gGraphics);
        if (null == fm)
        {
            size = getSize();
            fm = gGraphics.getFontMetrics();
            iWidth = fm.stringWidth(sText);
            iStartWidth = (size.width-iWidth)/2;
            iStartHeight = (size.height-fm.getHeight())/2+fm.getAscent();
        }
        gGraphics.setColor(getBackground());
        gGraphics.drawString(sText, iStartWidth, iStartHeight);
        if (bHasFocus)
        {
            gGraphics.drawLine(iStartWidth, iStartHeight, iStartWidth+iWidth, iStartHeight);
        }
    }


    public boolean isFocusTraversable()
    {
        return true;
    }

    public Dimension getPreferredSize()
    {
        if (0 == iWidth)
        {
            return super.getPreferredSize();
        }
        int iSide = 4*iBorderWidth + iWidth;
        return new Dimension(iSide, iSide);
    }

    public Dimension getMinimumSize()
    {
        if (0 == iWidth)
        {
            return super.getMinimumSize();
        }
        int iSide = 4*iBorderWidth + iWidth;
        return new Dimension(iSide, iSide);
    }

    public void focusGained(FocusEvent fe)
    {
        bHasFocus = true;
        repaint();
    }

    public void focusLost(FocusEvent fe)
    {
        bHasFocus = false;
        repaint();
    }    

  /**
   * Adds the specified action listener to receive action events
   * from this button.
   * @param listener the action listener
   */
   public void addActionListener(ActionListener listener)
   {
       actionListener = AWTEventMulticaster.add(actionListener, listener);
       enableEvents(AWTEvent.MOUSE_EVENT_MASK);
   }
 
   /**
    * Removes the specified action listener so it no longer receives
    * action events from this button.
    * @param listener the action listener
    */
   public void removeActionListener(ActionListener listener)
   {
       actionListener = AWTEventMulticaster.remove(actionListener, listener);
   }    


    private void actionButton()
    {
        if(actionListener != null) 
        {
            actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, sText));
        }
    }

    public void componentResized(ComponentEvent ce)
    {
        fm = null;
        repaint();
    }

    public void componentMoved(ComponentEvent ce)
    {
    }

    public void componentShown(ComponentEvent ce)
    {
    }

    public void componentHidden(ComponentEvent ce)
    {
    }

    public void mouseClicked(MouseEvent me)
    {
    }

    public void mousePressed(MouseEvent me)
    {
    }

    public void mouseReleased(MouseEvent me)
    {
        actionButton();
    }

    public void mouseEntered(MouseEvent me)
    {
    }

    public void mouseExited(MouseEvent me)
    {
    }

    public void keyPressed(KeyEvent ke)
    {
    }

    public void keyReleased(KeyEvent ke)
    {
        if ( (ke.getKeyCode() == KeyEvent.VK_ENTER) || (ke.getKeyCode() == KeyEvent.VK_SPACE) )
        {
            actionButton();
        }
    }

    public void keyTyped(KeyEvent ke)
    {
    }
}
