/*
 * @(#)TanSpinner.java	1.00 14th March 1999
 * 
 * Modification Log:
 *
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Panel;
import java.awt.Label;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.AWTEventMulticaster;
import java.awt.ItemSelectable;


/**
 * A spinner control.
 * Uses RoundButton and thus RoundLight controls.
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 14th March, 1999
 */

public class TanSpinner extends java.awt.Panel implements java.awt.event.ActionListener, java.awt.ItemSelectable
{
    private int             iValue, iMaxValue, iMinValue;
    private String          sParamName, sParamType;
    private RoundButton     butnDecr, butnIncr;
    private Label           lblStatus;
    private int[]           aiValues;
    private int             iValueIndex;
    ItemListener            itemListener;  // Post item events to listeners
    
    public TanSpinner(String sParamName, String sParamType, int iMaxValue, int iMinValue)
    {
        this.sParamName = sParamName;
        this.sParamType = sParamType;
        this.iMaxValue = iMaxValue;
        this.iMinValue = iMinValue;
        
        setLayout(new BorderLayout(2, 2));
        butnDecr = new RoundButton("<<");
        butnIncr = new RoundButton(">>");
        butnDecr.colorBorder = Color.black;
        butnDecr.setForeground(Color.yellow);
        butnDecr.addActionListener(this);
        butnIncr.colorBorder = Color.black;
        butnIncr.setForeground(Color.yellow);
        butnIncr.addActionListener(this);
        lblStatus = new Label("", Label.CENTER);
        setLabelText();
        
        add(butnDecr, BorderLayout.WEST);
        add(lblStatus, BorderLayout.CENTER);
        add(butnIncr, BorderLayout.EAST);
    }

    
    /**
     * Adds the specified listener 
     * @param listener the action listener
     */
    public void addItemListener(ItemListener listener)
    {
       itemListener = AWTEventMulticaster.add(itemListener, listener);
    }

   /**
    * Removes the specified listener so it no longer receives
    * @param listener the action listener
    */
    public void removeItemListener(ItemListener listener)
    {
        itemListener = AWTEventMulticaster.remove(itemListener, listener);
    }    
    
    public Object[] getSelectedObjects()
    {
        return null;
        
    }

    public void setValues(int [] aiValues)
    {
        this.aiValues = aiValues;
    }
    
    private void setLabelText()
    {
        lblStatus.setText("[ " + sParamName + " " + iValue + " " + sParamType + " ]");
        if (null != itemListener)
        {
            itemListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, Integer.toString(iValue), iValue));
        }
    }
    
    public int getValue()
    {
        return iValue;
    }
    
    public void setValue(int iValue)
    {
        this.iValue = iValue;
        setLabelText();
    }
    
    public void actionPerformed(ActionEvent ae)
    {
        if (null == aiValues)
        {
            if (ae.getSource().equals(butnIncr))
            {
                iValue++;
            }
            else if(ae.getSource().equals(butnDecr))
            {
                iValue--;
            }
        }
        else
        {
            if (ae.getSource().equals(butnIncr))
            {
                if (++iValueIndex >= aiValues.length)
                {
                    iValueIndex--;
                }
            }
            else if(ae.getSource().equals(butnDecr))
            {
                if (--iValueIndex < 0)
                {
                    iValueIndex++;
                }
            }
            iValue = aiValues[iValueIndex];
        }
        
        if (iValue > iMaxValue)
        {
            iValue = iMaxValue;
        }
        else if(iValue < iMinValue)
        {
            iValue = iMinValue;
        }
        else
        {
            setLabelText();
        }
    }
}
